@@ -146,7 +146,7 @@ PdfPainter::~PdfPainter()
         PdfError::LogMessage( eLogSeverity_Error, 
                               "PdfPainter::~PdfPainter(): FinishPage() has to be called after a page is completed!" );
 
-    assert(!m_pCanvas);
+    PODOFO_ASSERT( !m_pCanvas );
 }
 
 void PdfPainter::SetPage( PdfCanvas* pPage )
@@ -187,7 +187,7 @@ void PdfPainter::FinishPage()
 	try { 
 		if( m_pCanvas )
 			m_pCanvas->EndAppend();
-	} catch( const PdfError & e ) {
+	} catch( PdfError & e ) {
 	    // clean up, even in case of error
 		m_pCanvas = NULL;
 		m_pPage   = NULL;
@@ -947,7 +947,7 @@ void PdfPainter::DrawMultiLineText( double dX, double dY, double dWidth, double
     PdfString   sString  = this->ExpandTabs( rsText, rsText.GetCharacterLength() );
 
 	std::vector<PdfString> vecLines = GetMultiLineTextAsLines(dWidth, sString);
-
+    double dLineGap = m_pFont->GetFontMetrics()->GetLineSpacing() - m_pFont->GetFontMetrics()->GetAscent() + m_pFont->GetFontMetrics()->GetDescent();
     // Do vertical alignment
     switch( eVertical ) 
     {
@@ -962,13 +962,14 @@ void PdfPainter::DrawMultiLineText( double dX, double dY, double dWidth, double
             break;
     }
 
+    dY -= (m_pFont->GetFontMetrics()->GetAscent() + dLineGap / (2.0));
+
     std::vector<PdfString>::const_iterator it = vecLines.begin();
     while( it != vecLines.end() )
     {
         if( (*it).GetCharacterLength() )
-            this->DrawTextAligned( dX, dY - m_pFont->GetFontMetrics()->GetAscent(), dWidth, *it, eAlignment );
+            this->DrawTextAligned( dX, dY, dWidth, *it, eAlignment );
         dY -= m_pFont->GetFontMetrics()->GetLineSpacing();
-       
         ++it;
     }
     this->Restore();
@@ -993,11 +994,11 @@ std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const
     // fast and easy individual characters access    
     const std::string& stringUtf8 = rsText.GetStringUtf8();
     std::vector<pdf_utf16be> stringUtf16(stringUtf8.length() + 1, 0);
-	assert(stringUtf16.size() > 0);	
+    PODOFO_ASSERT( stringUtf16.size() > 0 );
     const pdf_long converted = PdfString::ConvertUTF8toUTF16(
 	    reinterpret_cast<const pdf_utf8*>(stringUtf8.c_str()), &stringUtf16[0], stringUtf16.size());
 	//const pdf_long len = rsText.GetCharacterLength();
-    assert( converted == (rsText.GetCharacterLength() + 1) );
+    PODOFO_ASSERT( converted == (rsText.GetCharacterLength() + 1) );
 
 	const pdf_utf16be* const stringUtf16Begin = &stringUtf16[0];
     const pdf_utf16be* pszLineBegin = stringUtf16Begin;
@@ -1014,7 +1015,7 @@ std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const
         {
             vecLines.push_back( PdfString( pszLineBegin, pszCurrentCharacter - pszLineBegin ) );
 
-            pszLineBegin = pszCurrentCharacter+1;// skip the line feed
+            pszLineBegin = pszCurrentCharacter + 1;// skip the line feed
             startOfWord = true;
             dCurWidthOfLine = 0.0;
         }
@@ -1024,7 +1025,20 @@ std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const
             {
                 // The previous word does not fit in the current line.
                 // -> Move it to the next one.
-                vecLines.push_back( PdfString( pszLineBegin, pszStartOfCurrentWord - pszLineBegin ) );
+                if( pszStartOfCurrentWord > pszLineBegin )
+                {
+                    vecLines.push_back( PdfString( pszLineBegin, pszStartOfCurrentWord - pszLineBegin ) );
+                }
+                else
+                {
+                    vecLines.push_back( PdfString( pszLineBegin, pszCurrentCharacter - pszLineBegin ) );
+                    // Skip all spaces at the end of the line
+                    while (IsSpaceChar(*(pszCurrentCharacter + 1)))
+                        pszCurrentCharacter++;
+
+                    pszStartOfCurrentWord = pszCurrentCharacter + 1;
+                    startOfWord=true;
+                }
                 pszLineBegin = pszStartOfCurrentWord;
 
                 if (!startOfWord)
@@ -1059,10 +1073,20 @@ std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const
                 {
                     // This word takes up the whole line.
                     // Put as much as possible on this line.                    
-                    vecLines.push_back( PdfString( pszLineBegin, pszCurrentCharacter - pszLineBegin ) );
-                    pszLineBegin = pszCurrentCharacter;
-                    pszStartOfCurrentWord = pszCurrentCharacter;
-                    dCurWidthOfLine = m_pFont->GetFontMetrics()->UnicodeCharWidth( SwapCharBytesIfRequired( *pszCurrentCharacter ) );
+                    if (pszLineBegin == pszCurrentCharacter)
+                    {
+                        vecLines.push_back( PdfString( pszCurrentCharacter, 1 ) );
+                        pszLineBegin = pszCurrentCharacter + 1;
+                        pszStartOfCurrentWord = pszCurrentCharacter + 1;
+                        dCurWidthOfLine = 0;
+                    }
+                    else
+                    {
+                        vecLines.push_back(PdfString(pszLineBegin, pszCurrentCharacter - pszLineBegin));
+                        pszLineBegin = pszCurrentCharacter;
+                        pszStartOfCurrentWord = pszCurrentCharacter;
+                        dCurWidthOfLine = m_pFont->GetFontMetrics()->UnicodeCharWidth( SwapCharBytesIfRequired( *pszCurrentCharacter ) );
+                    }
                 }
                 else
                 {
@@ -1070,7 +1094,7 @@ std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const
                     // -> Move it to the next one.                    
                     vecLines.push_back( PdfString( pszLineBegin, pszStartOfCurrentWord - pszLineBegin ) );
                     pszLineBegin = pszStartOfCurrentWord;
-                    dCurWidthOfLine = m_pFont->GetFontMetrics()->StringWidth( pszStartOfCurrentWord, (pszCurrentCharacter-pszStartOfCurrentWord) + 1 );
+                    dCurWidthOfLine = m_pFont->GetFontMetrics()->StringWidth( pszStartOfCurrentWord, (pszCurrentCharacter - pszStartOfCurrentWord) + 1);
                 }
             }
             else 
@@ -1083,7 +1107,7 @@ std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const
 
     if( (pszCurrentCharacter - pszLineBegin) > 0 ) 
     {
-        if( dCurWidthOfLine > dWidth )
+        if( dCurWidthOfLine > dWidth && pszStartOfCurrentWord > pszLineBegin )
         {
             // The previous word does not fit in the current line.
             // -> Move it to the next one.
@@ -1142,79 +1166,95 @@ void PdfPainter::DrawGlyph( PdfMemDocument* pDocument, double dX, double dY, con
         PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
     }
 
-	// search for a copy of this font to enter difference-encoding, create a new one if not found
-	PdfFont* pGlyphFont = pDocument->CreateDuplicateFontType1( m_pFont, "Glyph1" );
-
-	// select identical sizes
-	pGlyphFont->SetFontSize( m_pFont->GetFontSize() );
-	pGlyphFont->SetFontCharSpace( m_pFont->GetFontCharSpace() );
-	pGlyphFont->SetFontScale( m_pFont->GetFontScale() );
-	PdfObject* pGlyphFontObj = pGlyphFont->GetObject();
-
-	// get width of glyph
-	int width = static_cast<int>(pGlyphFont->GetFontMetrics()->GetGlyphWidth( pszGlyphname ) );
-
-	// change encoding to difference-encoding and adapt width
-	PdfObject* pEncoding = pGlyphFontObj->GetDictionary().GetKey( "Encoding" );
-
+	PdfFont* pGlyphFont = NULL;
 	int code = 32;
-	if ( pEncoding == NULL  ||  pEncoding->IsReference() == false )
+
+	for ( int num = 1; num <= 999; num++ )
 	{
+		// search for a copy of this font to enter difference-encoding, create a new one if not found
+		char suffix[256];
+		sprintf( suffix, "Glyph%i", num ); 
+		pGlyphFont = pDocument->CreateDuplicateFontType1( m_pFont, suffix );
+
+		PdfObject* pGlyphFontObj = pGlyphFont->GetObject();
+		PdfObject* pEncoding = pGlyphFontObj->GetDictionary().GetKey( "Encoding" );
+
 		// first time: create difference-encoding as reference, enter glyph
-		pEncoding = pDocument->GetObjects().CreateObject( "Encoding" );
-        
-		code++;
-        
-		PdfArray diffs;
-		diffs.push_back( PdfVariant( static_cast<pdf_int64>( code ) ) );
-		diffs.push_back( PdfName( pszGlyphname ) );
-        
-		pEncoding->GetDictionary().AddKey( "Differences", diffs );
-		pGlyphFontObj->GetDictionary().AddKey("Encoding", pEncoding->Reference() );
-
-		// clear Widths-array and enter width of this glyph
-		PdfObject* pWidthObj = pGlyphFontObj->GetIndirectKey( "Widths" );
-		PdfArray & rWidthArr = pWidthObj->GetArray();
-		for ( unsigned int i = 0; i < rWidthArr.size(); i++ )
-        {
-			rWidthArr[i] = PdfVariant( static_cast<pdf_int64>( 0 ) );
-        }
-        rWidthArr[code] = PdfVariant( static_cast<pdf_int64>( width ) );
-    }
-	else
-	{
-		// search glyph in existing Encoding/Difference, create if not found
-		pEncoding = pDocument->GetObjects().GetObject( pEncoding->GetReference() );
-        
-		PODOFO_ASSERT( pEncoding != NULL ); // paranoia
-
-		PdfArray diffs;
-		diffs = pEncoding->GetDictionary().GetKey( "Differences" )->GetArray();
-
-		bool foundIt = false;
-
-		TCIVariantList it = diffs.begin();
-		while( it != diffs.end() )
+		if ( pEncoding == NULL  ||  pEncoding->IsReference() == false )
 		{
-			if( (*it).GetDataType() == ePdfDataType_Name )
+
+			// get width of glyph to enter in difference-encoding
+			int width = static_cast<int>(pGlyphFont->GetFontMetrics()->GetGlyphWidth( pszGlyphname ) );
+			pEncoding = pDocument->GetObjects().CreateObject( "Encoding" );
+	        
+			code++;
+	        
+			PdfArray diffs;
+			diffs.push_back( PdfVariant( static_cast<pdf_int64>( code ) ) );
+			diffs.push_back( PdfName( pszGlyphname ) );
+	        
+			pEncoding->GetDictionary().AddKey( "Differences", diffs );
+			pGlyphFontObj->GetDictionary().AddKey("Encoding", pEncoding->Reference() );
+
+			// clear Widths-array and enter width of this glyph
+			PdfObject* pWidthObj = pGlyphFontObj->GetIndirectKey( "Widths" );
+			PdfArray & rWidthArr = pWidthObj->GetArray();
+			for ( unsigned int i = 0; i < rWidthArr.size(); i++ )
 			{
-				code++;
-				if ( (*it).GetName().GetName() == pszGlyphname )
-				{
-					foundIt = true;
-					break;
-				}
+				rWidthArr[i] = PdfVariant( static_cast<pdf_int64>( 0 ) );
 			}
-				
-			++it;
+			rWidthArr[code] = PdfVariant( static_cast<pdf_int64>( width ) );
+
+			break;
 		}
 
-		// TODO: if code exceeds 255, create a new font-copy and start again with code 33
-		PODOFO_ASSERT( code <= 255 );
 
-		if ( foundIt == false )
+		// Existing font, search for glyph in existing difference-encoding
 		{
+			pEncoding = pDocument->GetObjects().GetObject( pEncoding->GetReference() );
+	        
+			PODOFO_ASSERT( pEncoding != NULL ); // paranoia
+
+			PdfArray diffs;
+			diffs = pEncoding->GetDictionary().GetKey( "Differences" )->GetArray();
+
+			bool foundIt = false;
+
+			TCIVariantList it = diffs.begin();
+			while( it != diffs.end() )
+			{
+				if( (*it).GetDataType() == ePdfDataType_Name )
+				{
+					code++;
+					if ( (*it).GetName().GetName() == pszGlyphname )
+					{
+						foundIt = true;
+						break;
+					}
+				}
+					
+				++it;
+			}
+			if ( foundIt )	// glyph fount, use it
+				break;
+		}
+
+		// limit to codes <= 127, make new duplicate font if more
+		if ( code+1 >= 127 )
+		{
+			code = 32;
+			continue;
+		}
+
+		// add glyph to existing difference-encoding
+		{
+			// get width of glyph to enter in difference-encoding
+			int width = static_cast<int>(pGlyphFont->GetFontMetrics()->GetGlyphWidth( pszGlyphname ) );
+
 			code++;
+
+			PdfArray diffs;
+			diffs = pEncoding->GetDictionary().GetKey( "Differences" )->GetArray();
 			diffs.push_back( PdfName( pszGlyphname ) );
 
 			pEncoding->GetDictionary().AddKey( "Differences", diffs );
@@ -1223,10 +1263,20 @@ void PdfPainter::DrawGlyph( PdfMemDocument* pDocument, double dX, double dY, con
 			PdfObject* pWidthObj = pGlyphFontObj->GetIndirectKey( "Widths" );
 			PdfArray & rWidthArr = pWidthObj->GetArray();
 			rWidthArr[code] = PdfVariant( static_cast<pdf_int64>( width ) );
+
+			break;
 		}
 	}
 
-	if( m_pFont->IsSubsetting() ) {
+	// select identical sizes
+	pGlyphFont->SetFontSize( m_pFont->GetFontSize() );
+	pGlyphFont->SetFontCharSpace( m_pFont->GetFontCharSpace() );
+	pGlyphFont->SetFontScale( m_pFont->GetFontScale() );
+
+	PODOFO_ASSERT( code > 32  &&  code <= 127 );
+
+	if( m_pFont->IsSubsetting() ) 
+	{
 		// mark glyph as used in basefont (needed for subsetting)
 		m_pFont->AddUsedGlyphname( pszGlyphname );
 	}
